//#include <Arduino.h>
#include "include/Oz47.h"
#include "./libraries/SD/SD.h"
#include <include/RTClib.h>
#include <include/dht.h>
#include <include/Wire.h>
#include "/usr/lib/avr/include/avr/wdt.h"

/* Date: 03/02/2013
 * Copyright 2013 Pablo Cremades
 * Author: Pablo Cremades
 *  e-mail: pablocremades@gmail.com
 * Description:
 *
 * Changelog:
 * 19/03/2013: Inluded Timer1 Interrupt every 1ms for general purpose.
 *      Use Timer1 interrupt to make masurements every 5 minutes, instead of using a 5 minutes delay.
 *      Implementation of Watchdog to reset uController if something goes wrong. Have to check
 *      if bootloader supports this!
 * 20/03/2013: Modified setup() function so that MCU resets if SDCard inicialization is not possible.
 *      The MCU is also reset in case there is a problem opening file.
 *      Verified, the bootloader allows watchdog to be running. Must be Optiboot.
 * 17/02/2014: Add a second try in case dhtread() returns an error. It would be better to check the
 *      type of error.
 * 17/02/2014: Modified the previous retry logic. Now it retries reading dht while it gets TIMEOUT error.
 *      Be carefull!!!! Could stay there forever. Mmmm..., actually, the watchdog should solve the situation.
 */
/* *************************************************************************
*	This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.*
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY* without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* *************************************************************************/

#define      STATUS               8      //This pin indicates the status of the Ambient Monitor.

#define      DHT22_PIN            2
#define      PUMP                 3
#define      BattLev              A0
#define      SD_CS                10
#define      MTIME                300   //Tiempo entre mediciones en segundos.


//Creates the Ozone sensor Object.
Oz47 Ozone;

//Creates Temperature sensor Object.
dht DHT;

//Creates RTC Object.
RTC_DS1307 RTC;

//File
File Ozfile;

char  mode;

void setup()
{
    unsigned char sreg;     //For saving SREG content

    //Configures Timer1 interrupt.
    sreg = SREG;
    cli();
    TCCR1A &= ((1<<3) | (1<<2)); //Preserve only reserved bits.
    TCCR1B |= ((1<<WGM12) | (0<<CS12) | (1<<CS11) | (1<<CS10)); //Prescaler = clock/64
    TCCR1B &= ~((1<< CS12) | (0<<CS11) | (0<<CS10));
    TCCR1C |= (1<<FOC1A);
    OCR1A = 250;    //Con el prescaler se obtine f_ck = 16Mhz/64 = 250000. Contando hasta 250 obtenemos
    // una interrupcion cada 1ms.
    TIMSK1 |= (1 << OCIE1A); //Habilita interrupcion por timer1.
    SREG = sreg;    //Reestablece la configuracion general
    delay(1);
    sei();  //Activa las interrupciones globales.

    wdt_enable(WDTO_4S); //Set Watchdog to 4 seconds.

    pinMode(STATUS, OUTPUT);

    Serial.begin(19200);     //Set Ozone sensor serial interface.

    analogReference(EXTERNAL);

    Ozone.Mauto();

    Wire.begin(); //Starts I2C bus.
    RTC.begin();    //Starts RTC.

    pinMode(SD_CS, OUTPUT);     //Set SDCard CS pin.
    if ( !SD.begin(SD_CS) )
    {
        Serial.println("initialization failed!");
        TIMSK1 &= !(1 << OCIE1A);  //Stops Timer1 interrupts so that watchdog counter overflows and resets the uController.
        for (;;)    //Stay in endlessloop until watchdog resets the MCU.
        {
            digitalWrite(STATUS, HIGH);
            delay(300);
            digitalWrite(STATUS, LOW);
            delay(300);
        }
    }
}

static unsigned char domeasure = 0;
static int dhtStatus = 0;
void loop()
{
    //Check whether is time to measure.
    if( domeasure == 1)
    {
        domeasure = 0;
        String filename;  //Filename string.
        char dummybuf[4];        //Dummy buffer to convert numbers to strings.
        DateTime now = RTC.now();   //Initialize object "now" with RTC time.
        //Build filename from date.
        sprintf(dummybuf, "%02d", now.day());    //Force "day" to 2 digits format.
        filename += dummybuf;
        sprintf(dummybuf, "%02d", now.month());  //Force "month" to 2 digits format.
        filename += dummybuf;
        filename += now.year();
        filename += ".txt";

        delay(10);
        dummybuf[0] = 0;
        if( !SD.exists(&filename[0]) )
            dummybuf[0] = 1;    //Remember that the file is new for later use.
        else
            dummybuf[0] = 0;

        Ozfile = SD.open(&filename[0], FILE_WRITE | O_APPEND);  //Open file for Append. Create if it doesn't exist.

        if( !Ozfile )
        {
            Serial.println("Couldn't open file.");
            TIMSK1 &= !(1 << OCIE1A);  //Stops Timer1 interrupts so that watchdog counter overflows and resets the uController.
            for(;;)   //Endless loop if error occurs.
            {
              digitalWrite(STATUS, HIGH);
              delay(500);
              digitalWrite(STATUS, LOW);
              delay(100);
            }
        }
        else
        {
            cli(); //Stop interrupts to avoid problems. Check that the time it takes to perform the following tasks
                    //don't excede watchdog time.-------------------------------------------------------------------------------------------->
            if( dummybuf[0] == 1)
                Ozfile.println("Day,Month,Year,Hr,Mn,T,RH,Resist,Oz,Toz47,RHoz47,VBatt");
            sprintf(dummybuf, "%02d,", now.day());    //Force "day" to 2 digits format.
            Ozfile.print(dummybuf);
            sprintf(dummybuf, "%02d,", now.month());  //Force "month" to 2 digits format.
            Ozfile.print(dummybuf);
            Ozfile.print(now.year());
            Ozfile.print(',');
            sprintf(dummybuf, "%02d,", now.hour());  //Force "hour" to 2 digits format.
            Ozfile.print(dummybuf);
            sprintf(dummybuf, "%02d,", now.minute());  //Force "minute" to 2 digits format.
            Ozfile.print(dummybuf);

            dhtStatus = DHT.read22(DHT22_PIN);
            while( dhtStatus == DHTLIB_ERROR_TIMEOUT ){        //Retry in case of error. Carefull!!! Could be here forever!!!. Mmmm, actually
              delay(100);                                      //the watchdog should solve the situation.
              dhtStatus = DHT.read22(DHT22_PIN);
            }

            switch(dhtStatus)
            {
            case DHTLIB_OK:
                Ozfile.print(DHT.temperature);
                Ozfile.print(',');
                Ozfile.print(DHT.humidity);
                Ozfile.print(',');
                break;
            default:
                Ozfile.print(-999);
                Ozfile.print(',');
                Ozfile.print(-999);
                Ozfile.print(',');
            }
            Ozone.Sense();
            Ozfile.print(Ozone.getRes());
            Ozfile.print(',');
            Ozfile.print(Ozone.getOz());
            Ozfile.print(',');
            Ozfile.print(Ozone.getTemp());
            Ozfile.print(',');
            Ozfile.print(Ozone.getRH());
            Ozfile.print(',');
            Ozfile.print(analogRead(BattLev));
            Ozfile.println("");
            Ozfile.close();
            sei();
        }
    }
    //Do anything else.
}

/* Interrupcion por timer CTC. Ocurre cada 1ms*/
/* Parpadea el led conectado al pin 8*/
ISR(TIMER1_COMPA_vect)
{
    /* Interrupccion cada 1ms */
    static unsigned int mseconds = 0;
    static unsigned int seconds = 0;
    wdt_reset();    //Reset Watchdog.
    mseconds++;
    //Espera a que se cumpla 1 segundo.
    if( mseconds > (1000 - 1))
    {
        mseconds = 0;
        PORTB ^= (1<<PB0);      //PB0 is the Status LED pin. This will make the led blink every 1 second.
        seconds++;
    }
    if(seconds > MTIME - 1)
    {
        seconds = 0;
        domeasure = 1;  //Set this flag to tell main loop to perform a measurement.
    }

}
